function [Wx,Wy,Xm,Ym]=cca(X,Y)
%CCA  Canonical Correlation Analysis.
%  [Wx,Wy,Xm,Ym]=cca(X,Y) calculates canonical corelation vectors Wx and Wy from
%  two sets of observations X and Y. It returns also the mean values 
%  of X and Y (Xm and Ym). The number of observations in X and in Y has to be equal.
%  All vectors are given in columns of corresponding matrices.
%  As a result min(p,q,N) pairs of canonical correlation vectors are returned, 
%  where p and q are the sizes of vectors in X and Y and N is the number of observations.
%  Where p>N or q>N the dual formulation of CCA is used to speed up calculation.
%  This is a basic procedure for CCA, withouth regularization.

[p,N]=size(X);
[q,N]=size(Y);
k=min([p q N]);

%center data around the mean
Xm=mean(X,2);
X=X-repmat(Xm,1,N);
Ym=mean(Y,2);
Y=Y-repmat(Ym,1,N);

%construct A and B
if p<=N & q<=N
   Cxx=X*X';
   Cyy=Y*Y';
   Cxy=X*Y';
   Cyx=Y*X';
   A=[zeros(p,p) Cxy; Cyx, zeros(q,q)];
   B=[Cxx zeros(p,q); zeros(q,p) Cyy];
elseif p>N & q>N
   Cxx=X'*X;
   Cyy=Y'*Y;
   A=[zeros(N,N) Cxx*Cyy; Cyy*Cxx, zeros(N,N)];
   B=[Cxx*Cxx zeros(N,N); zeros(N,N) Cyy*Cyy];
elseif p>=N & q<=N   
   Cxx=X'*X;
   Cyy=Y*Y';
   A=[zeros(N,N) Cxx*Y'; Y*Cxx, zeros(q,q)];
   B=[Cxx*Cxx zeros(N,q); zeros(q,N) Cyy];
else %p<=N & q>=N   
   Cxx=X*X';
   Cyy=Y'*Y;
   A=[zeros(p,p) X*Cyy; Cyy*X', zeros(N,N)];
   B=[Cxx zeros(p,N); zeros(N,p) Cyy*Cyy];
end   
   
%generalized eigendecomposition
[Z,d]=eig(A,B);

%sort eigenvalues
d=diag(d);
[ds,sidxs]=sort(real(-d));
rsidxs=sidxs([abs(imag(ds))<10^-10]);
Z=Z(:,rsidxs(1:k));

%map to input spaces
if p<=N & q<=N
   Wx=Z(1:p,:);
   Wy=Z(p+1:end,:);
elseif p>N & q>N
   Wx=X*Z(1:N,:);
   Wy=Y*Z(N+1:end,:);
elseif p>=N & q<=N   
   Wx=X*Z(1:N,:);
   Wy=Z(N+1:end,:);
else %p<=N & q>=N   
   Wx=Z(1:p,:);
   Wy=Y*Z(p+1:end,:);
end   
