function A=is2fsRob(X,Xm,U,L,k,r,s,nh)
%IS2FSROB  Robust maping from image space to feature space.
%   A = is2fsRob(X,Xm,U,k,r,s,nh) maps the image(s) in the vector(s) X from 
%   the image space to the feature space in a robust manner. Xm is the mean image, 
%   U are the basis vectors of the feature space. k,r, and s are the parameters
%   of the robust method. nh is the number of hypotheses to perform.
%   A is the calculated vector(s) in the feature space. 

%default parameters
if nargin<5 k=48;r=.85;s=24; end;
if nargin<8 nh=5; end;

[M,K]=size(U);
N=size(X,2);

A=zeros(K,N);
Acand=zeros(K,nh);
for j=1:N 
    for i=1:nh  %generate hypos        
        Acand(:,i)=genHypo(X(:,j),Xm,U,k,r,s);
    end;
    A(:,j)=selCoef(X(:,j),Xm,U,Acand,L);  %select the best hypothesis        
end;


%-------------------------------------------------------------------------------------------%

function a = genHypo(X,Xm,U,k,r,s)
% generates one hypothesis
% X ... image vector
% U ... eigenvectors
% k ... k*K is the initial number of selected pixels
% r ... reduction factor in alpha trimming
% s ... s*K is stopping criterion 
% a ... resulting coefficient vector

[M,K] = size(U);
if nargin < 7
   init_inds = 1:M;
end
Xd=X-Xm;

s_points = ceil(M*rand(1,k*K))';
nbp=length(s_points);
   
B = Xd(s_points);
A = U(s_points, :);
points = 1:nbp;
sol = A\B;
err = abs(A*sol - B);

while (nbp > s*K)               %alpha trimming 
   errh = sort(err);
   err_marg = errh(ceil(r*nbp));
   points = points(find((err <= err_marg)));
   s_points = s_points(points);
   nbp = length(points);
   B = B(points);
   A = A(points, :);
   points = 1:nbp;
   sol = A\B;
   err = abs(A*sol - B);
end;

a = sol;


%-------------------------------------------------------------------------------------------%


function a=selCoef(X,Xm,U,Acand,L)
% Simple selection method - winner takes all
% X ... image vector
% Xm ... mean vector
% U ... eigenvectors
% Acand ... candidate coefficient vectors
% a ... the selected coefficient vector

k1=1;
k2=.01;

[K,nh]=size(Acand);
N=length(L);
M=size(X,1);
Xd=X-Xm;
q=zeros(nh,1);

Y=U*Acand;   % reconstruct images with the coeffs
err_marg=2*sum(L(K+1:N))/M;  %estimate expected error
for i=1:nh     
  err=(Y(:,i)-Xd).^2;  %calculate reconstruction error
  points=find((err<err_marg));  %find compatible points
  q(i)=k1*length(points)-k2*sum(err(points));  
end;

[q,ind]=sort(q);
i_sel=ind(nh);
a=Acand(:,i_sel);  %selected coefficient vector 

